package de.schloenvoigt.sparqltesttool.model;

/********************************************************************

This file is part of SPARQLTestTool - A Tool for testing SPARQL Queries

Copyright (C) 2010 Steffen Schlönvoigt <steffen@schloenvoigt.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 	
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 	
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/ 

import java.util.ArrayList;
import java.util.prefs.Preferences;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A Model for the SPARQL Test Tool 
 */
public class SPARQLTestToolModel {
	
// ---------------------------- ATTRIBUTES ----------------------------------
	/**
	 * This string represents the data
	 */
	private String dataString = "";
	/**
	 * This String stores the query
	 */
    private String queryString = "";
    /**
     * This String stores the output
     */
    private String outputString = "";

// ----------------------------- ACCESSORS ----------------------------------    

    /**
     * Returns the current data String
     * @return a String
     */
    public String getDataString( )
    {
        return this.dataString;
    }
    
    /**
     * Returns the current query String
     * @return a String
     */
    public String getQueryString( )
    {
        return this.queryString;
    }
    
    /**
     * Returns the current output String
     * @return a String
     */
    public String getOutputString( )
    {
        return this.outputString;
    }

    /**
     * Sets the Texts of the input fields
     * @param data	   The string representing the data
     * @param query   The string representing the query
     */
    public void setInputFieldTexts( String data, String query )
    {
    	this.dataString  = data;
    	this.queryString = query;
    }
    
    /**
     * Sets the output String
     */
    public void setOutputString( String output )
    {
    	this.outputString = output;
    	this.notifyListeners();
    }
    
// -------------------- CONSTRUCTION/DESTRUCTION ----------------------------
    
    /**
     * CONSTRUCTOR - Create/Load the Model
     */
    public SPARQLTestToolModel( )
    {
    	// Get stored values from last session
    	this.loadFromLocalStore();
    }
    
    /**
     * This method is called on exit
     */
    public void finalize( )
    {
    	this.saveToLocalStore();
    }
    
    
    
//---------- Facility to load contents of last session on startup -----------     
    
    /**
    * Loads the contents of the Fields from local data
    */
    private void loadFromLocalStore( )
    {
    	try{
	    	Preferences userPrefs = Preferences.userNodeForPackage( this.getClass() );
	    	this.dataString  = userPrefs.get( "DATA", "" );
	    	this.queryString = userPrefs.get( "QUERY", "" );
    	}catch( Exception x ){ /* Just ignore */ }
    }
    
    /**
    * Stores the contents of the fields in the local data
    */
    public void saveToLocalStore( )
    {
    	try{    	
	    	Preferences userPrefs = Preferences.userNodeForPackage( this.getClass() );
	    	userPrefs.put( "DATA",  this.dataString );
	    	userPrefs.put( "QUERY", this.queryString );
    	}catch( Exception x ){ /* Just ignore */ }
    }
    
//------------------- ChangeListener Handling -------------------------------    
    
    /**
     *  The list of ChangeListeners that are registered at this Model
     */
    private ArrayList<ChangeListener> changeListeners = new ArrayList<ChangeListener>();
    
    /**
     *  Notify all registered ChangeListeners
     */
    private void notifyListeners( )
    {
    	ChangeEvent e = new ChangeEvent( this );
    	for( ChangeListener cl : changeListeners )
    	{
    		cl.stateChanged( e );
    	}
    }
    
    /**
     * Register a ChangeListener for this Model which will from 
     * then on get notified about changes
     * @param cl a ChangeListener
     */
    public void addChangeListener( ChangeListener cl )
    {
    	if( ! this.changeListeners.contains( cl ) )
    	{
    		this.changeListeners.add( cl );
    	}
    }
    
    /**
     * Unregisters a ChangeListener from this Model
     * @param cl a ChangeListener
     */
    public void removeChangeListener( ChangeListener cl )
    {
    	if( this.changeListeners.contains( cl ) )
    	{
    		this.changeListeners.remove( cl );
    	}
    }
    

	

}
