package de.schloenvoigt.sparqltesttool.ui;

/********************************************************************

This file is part of SPARQLTestTool - A Tool for testing SPARQL Queries

Copyright (C) 2010 Steffen Schlönvoigt <steffen@schloenvoigt.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 	
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 	
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/ 

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.schloenvoigt.sparqltesttool.control.SPARQLTestToolController;
import de.schloenvoigt.sparqltesttool.model.SPARQLTestToolModel;

/**
 * Main Frame of the Application
 * @author steffen schloenvoigt steffen@schloenvoigt.de
 * 2010-01-27
 */
public class SPARQLTestToolMainFrame extends JFrame implements ChangeListener, WindowListener {

// -------------------------- ATTRIBUTES ------------------------------------
		
	private static final long serialVersionUID = -5801096359807167684L;
	private JTextArea   dataInputArea;
	private JTextArea   queryInputArea;
	private JEditorPane outputArea;
	
	private JSplitPane  mainPanel;	
	private JSplitPane	 queryPanel;	
	private JPanel      actionPanel;
	
	private JScrollPane dataScrollPane;
	private JScrollPane queryScrollPane;
	private JScrollPane outputScrollPane;
	
	private JButton     queryButton;
	
	protected SPARQLTestToolController controller;
	protected SPARQLTestToolModel      model;

// --------------------------- ACCESSORS ------------------------------------
	
	/**
	 * Sets the controller, to which the view delegates actions 
	 * @param controller
	 */
	public void setController( SPARQLTestToolController controller )
	{
		this.controller = controller;
	}

	/**
	 * Sets the model for which we should display the data
	 * @param model
	 */
	public void setModel ( SPARQLTestToolModel model )
	{
		if( this.model != null )this.model.removeChangeListener( this );
		this.model = model;
		this.readModel();
		this.model.addChangeListener( this );
		
	}
// ------------------------- INITIALIZATION ---------------------------------	
	
	/**
	 * Creates a new MainFrame
	 * @throws HeadlessException
	 */
	public SPARQLTestToolMainFrame( ) throws HeadlessException 
	{		
		this.setTitle( "SPARQL Test Tool" );
		//this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE ); 
		this.addWindowListener( this );
	    this.setSize( 1000, 600 );	    
	    this.initializeComponent();
	    this.setLocationRelativeTo( null );
	    this.setVisible( true ); 
	    mainPanel.setDividerLocation( 0.5 );
		queryPanel.setDividerLocation( 0.3 );
		
	}	
	
	/**
	 * Initializes the Frame
	 */
	private void initializeComponent( )
	{
		// First initialize the components
		dataInputArea    = new JTextArea();
		queryInputArea   = new JTextArea();
		outputArea       = new JEditorPane();
		
		mainPanel        = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );		
		queryPanel       = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		
		actionPanel      = new JPanel();		
		queryButton      = new JButton( new QueryAction() );
		
		dataScrollPane   = new JScrollPane();
		queryScrollPane  = new JScrollPane();
		outputScrollPane = new JScrollPane();
		
		// Now configure the components
		Font monospaceFont = new Font( Font.MONOSPACED, Font.PLAIN, 12 );
		dataInputArea.setBorder( BorderFactory.createTitledBorder("Data Input") );
		dataInputArea.setFont( monospaceFont );
		
		queryInputArea.setBorder( BorderFactory.createTitledBorder( "Query Input" ) );
		queryInputArea.setFont( monospaceFont );
		outputArea.setBorder( BorderFactory.createTitledBorder( "Output" ) );		
		outputArea.setEditable( false );
		outputArea.setContentType( "text/html" );		
				
		// Now put everything together
		dataScrollPane.setViewportView( dataInputArea );
		queryScrollPane.setViewportView( queryInputArea );
		outputScrollPane.setViewportView( outputArea );
		
		queryPanel.setTopComponent( queryScrollPane );
		queryPanel.setBottomComponent( outputScrollPane );
		
		mainPanel.setLeftComponent( dataScrollPane );
		mainPanel.setRightComponent( queryPanel );
		
		actionPanel.add( queryButton );
		
		this.add( mainPanel, BorderLayout.CENTER );
		this.add( actionPanel, BorderLayout.SOUTH );
		
		
		
	}
	
// ---------------- UPDATING THE DISPLAYED INFORMATION ---------------------- 

	/**
	 * Reads the passed model and displays its data
	 * @param model a Model that contains values that should be displayed
	 */
	private void readModel(  )
	{
		this.dataInputArea.setText(  this.model.getDataString() );
		this.queryInputArea.setText( this.model.getQueryString() );
		this.outputArea.setText(     this.model.getOutputString() );
	}

	@Override
	public void stateChanged(ChangeEvent e) 
	{
		if( e.getSource() instanceof SPARQLTestToolModel &&
		    e.getSource().equals( this.model ) )
		{
			this.readModel( );
		}				
	}
	
// -------------------------- Action Implementations ------------------------	
	
	/**
	 * Defines the Model/Behaviour of the Query Button
	 */
	private class QueryAction extends AbstractAction
	{		
		private static final long serialVersionUID = 1L;

		public QueryAction( )
		{
			this.putValue(NAME, "Query");
		}

		@Override
		public void actionPerformed(ActionEvent e) 
		{
			if( controller != null )
			{
				controller.query( dataInputArea.getText(), queryInputArea.getText() );
			}
		}
		
	}

// ----------------------------- WINDOW LISTENER ----------------------------

	@Override
	public void windowClosing(WindowEvent e) 
	{
		// Tell the Model, that we are closing now
		this.model.finalize();
		System.out.println( "ByeBye" );
		System.exit( 0 );
	}

	@Override
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosed(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	
	@Override
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

}
